;;;; fps.scm - a faked syntax-case module

(cond-expand
 (syntax-case
     (require 'fps-base)

     (module fps
	 (;; constants
          pi 1/4pi 1/2pi 3/4pi 5/4pi 3/2pi 7/4pi 2pi
	     origin
	     identity-matrix
	     afm-directory-list

	     ;; font
	     font?
	     font

	     ;; Point and Matrix
	     pt? pt=
	     pt pt:x pt:y
	     add-pts  negate-pt  scale-pt

	     matrix? matrix=
	     matrix matrix*

	     ;; Path Makers
	     path?
	     line 
	     rect
	     arc 
	     tangent-arc 
	     curve 
	     close-path 
	     stroke-outline-path 
	     bitmap->path
	     bounding-box->rect
	     the-empty-path

	     ;; glyphs construction
	     char->glyphpath
	     int->glyphpath
	     glyphname->glyphpath
	     vector->glyphpath
	     simple-string->glyphpath
	     string->glyphpath

	     ;; Picture Makers
	     picture?
	     stroke fill clip colormap bitmap->pict
	     paint-glyphpath
	     the-empty-pict

	     ;; combination
	     compose       compose-path   compose-pict
	     join          join-path      join-pict
	     link

	     ;; transformation
	     translate rotate scale

	     ;; style
	     style?
	     vary-default
	     build-style	  
	     with-style*
	     with-style
	     with-attrib

	     ;; attributes
	     attrib?
	     :color        :line-cap     :line-width 
	     :dash-pattern :dash-offset  
	     :line-join    :miter-limit

	     ;; colors
	     color?   color=
	     gray     gray:val
	     rgb      rgb:r  rgb:g  rgb:b
	     hsb      hsb:h  hsb:s  hsb:b
	     cmyk     cmyk:c cmyk:m cmyk:y cmyk:k

	     ;; char map
	     char-map?
	     base-char-map
	     lookup-char-map
	     function->char-map 
	     alist->char-map 
	     mask-char-map 
	     native-font-char-map

	     ;; int map
	     int-map?
	     base-int-map
	     lookup-int-map
	     function->int-map
	     alist->int-map
	     mask-int-map
	     native-font-int-map

	     ;; object info
	     start-pt      
	     end-pt
	     bounding-box  
	     bounding-box:max bounding-box:min

	     ;; channel
	     channel?
	     show
	     show-w/ps2-text-channel
	     ps2-text-channel
	     close-channel

	     ;; bitmap
	     bitmap?
	     vector->bitmap
	     hex-string->bitmap
	     bin-string->bitmap

	     ;; options
	     :format
	     :creator      :creation-date  :title
	     :copyright    :for            :routing
	     :duplex       :duplex-tumble  :collate  
	     :num-copies   :orientation
	     :page-label

	     ;; util
	     deg->rad rad->deg
	     inch
	     )

       (define-syntax with-style
	 (syntax-rules ()
	   ((with-style style exp ...)
	    (fps:with-style* style (lambda () exp ...)))))
       
       (define-syntax with-attrib
	 (syntax-rules ()
	   ((with-attrib (attrib ...) exp ...)
	    (with-style (apply fps:build-style (fps:default-style) (list attrib ...))
			exp ...))))
       
       (define pi fps:pi) (define 1/4pi fps:1/4pi) (define 1/2pi fps:1/2pi) (define 3/4pi fps:3/4pi)
       (define 5/4pi fps:5/4pi) (define 3/2pi fps:3/2pi) (define 7/4pi fps:7/4pi) (define 2pi fps:2pi)
       (define origin fps:origin)
       (define identity-matrix fps:identity-matrix)
       (define afm-directory-list fps:afm-directory-list)

       ;; font
       (define font? fps:font?)
       (define font fps:font)

       ;; Point and Matrix
       (define pt? fps:pt?) (define pt= fps:pt=)
       (define pt fps:pt) (define pt:x fps:pt:x) (define pt:y fps:pt:y)
       (define add-pts fps:add-pts)  (define negate-pt fps:negate-pt)  (define scale-pt fps:scale-pt)

       (define matrix? fps:matrix?) (define matrix= fps:matrix=)
       (define matrix fps:matrix) (define matrix* fps:matrix*)

       ;; Path Makers
       (define path? fps:path?)
       (define line fps:line) 
       (define rect fps:rect)
       (define arc fps:arc) 
       (define tangent-arc fps:tangent-arc) 
       (define curve fps:curve) 
       (define close-path fps:close-path) 
       (define stroke-outline-path fps:stroke-outline-path) 
       (define bitmap->path fps:bitmap->path)
       (define bounding-box->rect fps:bounding-box->rect)
       (define the-empty-path fps:the-empty-path)

       ;; glyphs construction
       (define char->glyphpath fps:char->glyphpath)
       (define int->glyphpath fps:int->glyphpath)
       (define glyphname->glyphpath fps:glyphname->glyphpath)
       (define vector->glyphpath fps:vector->glyphpath)
       (define simple-string->glyphpath fps:simple-string->glyphpath)
       (define string->glyphpath fps:string->glyphpath)

       ;; Picture Makers
       (define picture? fps:picture?)
       (define stroke fps:stroke) (define fill fps:fill) (define clip fps:clip) (define colormap fps:colormap) (define bitmap->pict fps:bitmap->pict)
       (define paint-glyphpath fps:paint-glyphpath)
       (define the-empty-pict fps:the-empty-pict)

       ;; combination
       (define compose fps:compose)       (define compose-path fps:compose-path)   (define compose-pict fps:compose-pict)
       (define join fps:join)          (define join-path fps:join-path)      (define join-pict fps:join-pict)
       (define link fps:link)

       ;; transformation
       (define translate fps:translate) (define rotate fps:rotate) (define scale fps:scale)

       ;; style
       (define style? fps:style?)
       (define vary-default fps:vary-default)
       (define build-style fps:build-style)	  
       (define with-style* fps:with-style*)

       ;; attributes
       (define attrib? fps:attrib?)
       (define :color fps::color)        (define :line-cap fps::line-cap)     (define :line-width fps::line-width) 
       (define :dash-pattern fps::dash-pattern) (define :dash-offset fps::dash-offset)  
       (define :line-join fps::line-join)    (define :miter-limit fps::miter-limit)

       ;; colors
       (define color? fps:color?)   (define color= fps:color=)
       (define gray fps:gray)     (define gray:val fps:gray:val)
       (define rgb fps:rgb)      (define rgb:r fps:rgb:r)  (define rgb:g fps:rgb:g)  (define rgb:b fps:rgb:b)
       (define hsb fps:hsb)      (define hsb:h fps:hsb:h)  (define hsb:s fps:hsb:s)  (define hsb:b fps:hsb:b)
       (define cmyk fps:cmyk)     (define cmyk:c fps:cmyk:c) (define cmyk:m fps:cmyk:m) (define cmyk:y fps:cmyk:y) (define cmyk:k fps:cmyk:k)

       ;; char map
       (define char-map? fps:char-map?)
       (define base-char-map fps:base-char-map)
       (define lookup-char-map fps:lookup-char-map)
       (define function->char-map fps:function->char-map) 
       (define alist->char-map fps:alist->char-map) 
       (define mask-char-map fps:mask-char-map) 
       (define native-font-char-map fps:native-font-char-map)

       ;; int map
       (define int-map? fps:int-map?)
       (define base-int-map fps:base-int-map)
       (define lookup-int-map fps:lookup-int-map)
       (define function->int-map fps:function->int-map)
       (define alist->int-map fps:alist->int-map)
       (define mask-int-map fps:mask-int-map)
       (define native-font-int-map fps:native-font-int-map)

       ;; object info
       (define start-pt fps:start-pt)      
       (define end-pt fps:end-pt)
       (define bounding-box fps:bounding-box)  
       (define bounding-box:max fps:bounding-box:max) (define bounding-box:min fps:bounding-box:min)

       ;; channel
       (define channel? fps:channel?)
       (define show fps:show)
       (define show-w/ps2-text-channel fps:show-w/ps2-text-channel)
       (define ps2-text-channel fps:ps2-text-channel)
       (define close-channel fps:close-channel)

       ;; bitmap
       (define bitmap? fps:bitmap?)
       (define vector->bitmap fps:vector->bitmap)
       (define hex-string->bitmap fps:hex-string->bitmap)
       (define bin-string->bitmap fps:bin-string->bitmap)

       ;; options
       (define :format fps::format)
       (define :creator fps::creator)      (define :creation-date fps::creation-date)  (define :title fps::title)
       (define :copyright fps::copyright)    (define :for fps::for)            (define :routing fps::routing)
       (define :duplex fps::duplex)       (define :duplex-tumble fps::duplex-tumble)  (define :collate fps::collate)  
       (define :num-copies fps::num-copies)   (define :orientation fps::orientation)
       (define :page-label fps::page-label)

       ;; util
       (define deg->rad fps:deg->rad) (define rad->deg fps:rad->deg)
       (define inch fps:inch)
       ) ) 

 (hygienic-macros

  (define-syntax with-style
    (syntax-rules ()
      ((with-style style exp ...)
       (fps:with-style* style (lambda () exp ...)))))
  
  (define-syntax with-attrib
    (syntax-rules ()
      ((with-attrib (attrib ...) exp ...)
       (with-style (apply fps:build-style (fps:default-style) (list attrib ...))
		   exp ...)))) 
  )

 (else

  (define-macro (with-style style . exp)
    `(fps:with-style* ,style (lambda () ,@exp)))

  (define-macro (with-attrib attribs . exp)
    `(with-style (apply fps:build-style (fps:default-style) (list ,@attribs))
		 ,@exp) ) )
 
 )
